import socket , sys , signal
from Node import Node
from threading import Thread
from clint.textui import colored

class Client(Node):

	def __init__(self , host , port , passwd):
		self.host = host
		self.port = port
		self.passwd = passwd + '\n'
		self.sock = socket.socket(socket.AF_INET , socket.SOCK_STREAM)

	def start(self):
		self.sock.connect((self.host , self.port))
		self.sendMsg(self.sock ,self.passwd)
		message = self.recvMsg(self.sock , '\n')
		if message.replace('\n' , '')!='~q':
			username = raw_input(colored.yellow("Enter your username: "))
			Thread(target=self.sends , args = (username,)).start()
			Thread(target=self.recvs , args=('\n',)).start()
		else:
			sys.exit(0)

	def sends(self , username):
		while True: 
			msg = raw_input()
			# signal.signal(signal.SIGINT , signal_handler)
			print "\033[A                             \033[A"
			print colored.yellow("<me>") , colored.magenta(msg)
			if msg=='~q':
				msg+='\n'
				self.sendMsg(self.sock , msg)
				self.sock.shutdown(socket.SHUT_WR)
				break
			msg+='\n'
			msg = '<' + username + '> ' + msg
			self.sendMsg(self.sock , msg)
	def recvs(self , delimeter):
		while True:
			msg = self.recvMsg(self.sock , delimeter)
			if msg.replace('\n' , '')=='~q':
				# self.sock.shutdown(socket.SHUT_RD)
				self.sock.close()
				break
			msg =  msg.replace('\n' , '')
			ind = msg.index('>')
			print colored.yellow(msg[0:ind+1]) , colored.red(msg[ind+1:])

def main(argv):
	client = Client(argv[0] , 1060 , argv[1])
	client.start()


class Node(object):
	
	def sendMsg(self , sock , message):
		sock.sendall(message)

	def recvMsg(self, sock , delimeter):
		message = ""
		while True:
			data = sock.recv(4096)
			if not data:
				break
			message+=data
			if delimeter in data:
				break
		return message


import socket, sys , signal
from threading import Thread
from Node import Node


class Server(Node):

	def __init__(self , host , port , passwd):
		self.host = host
		self.port = port
		self.passwd = passwd
		self.sock = socket.socket(socket.AF_INET , socket.SOCK_STREAM)
		self.sock.setsockopt(socket.SOL_SOCKET , socket.SO_REUSEADDR , 1)
		self.registered_clients = []

	def start(self):
		self.sock.bind((self.host , self.port))
		self.sock.listen(3)
		def chat(sc):		
			while True:
				message = self.recvMsg(sc , '\n')
				if not message:
					break
				if message.replace('\n' , '')=='~q':
					self.sendMsg(sc , message)
					break
				print message.replace('\n' , '')
				self.sendToAllClients(message , sc)
				#Later will have to add a method to escape \n in case it is already present in the message.
			self.deregisterClient(sc)
			sc.close()
		while True:
			sc , peeraddr = self.sock.accept()
			msg = self.recvMsg(sc , '\n')
			if msg.replace('\n' , '') == self.passwd:
				self.sendMsg(sc , 'q\n')
				self.registerClient(sc)
				Thread(target=chat , args=(sc,)).start()
			else:
				self.sendMsg(sc , '~q\n')
				sc.close()

	def registerClient(self , sc):
		if sc not in self.registered_clients:
			self.registered_clients.append(sc)

	def sendToAllClients(self , msg , sc):
		for sock in self.registered_clients:
			if sock is not sc:
				sock.sendall(msg)

	def deregisterClient(self , sc):
		if sc in self.registered_clients:
			self.registered_clients.remove(sc)

def main(argv):
	server = Server(argv[0] , 1060 , argv[1])
	try:
		server.start()
	except KeyboardInterrupt:
		print "Ok! stopping the server!"
		sys.exit(0)


try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup

setup(
	name = 'tcp_chat' , 
	version = '0.1.1',
	description = 'A chat app built over TCP',
	author = 'Punit Dhoot' , 
	author_email = 'punitdhoot1@gmail.com',
	url = 'https://github.com/pdhoot/tcp_chat' ,
	license='MIT',
	packages = ['tcp_chat'],
	install_requires = ['clint'],
	download_url = 'https://github.com/pdhoot/tcp_chat/tarball/0.1' ,
	keywords = ['chat' , 'tcp-chat'],
	entry_points = {
		'console_scripts' : ['tcp_chat = tcp_chat.tcp_chat:main']
	}
)


import argparse
import server 
import client

def main():
	desc = "A chat app built over TCP"
	parser = argparse.ArgumentParser(description = desc)
	group = parser.add_mutually_exclusive_group()
	group.add_argument('-s' , '--server' , action='store_true' , help = 'starts the server')
	group.add_argument('-c' , '--client' , action='store_true' , help = 'starts the client')
	parser.add_argument('interface')
	parser.add_argument('password')
	args = parser.parse_args()
	argv = [args.interface , args.password]

	if args.server== True:
		server.main(argv)
	elif args.client== True:
		client.main(argv)
	else:
		parser.print_help()